@@ -62,29 +62,43 @@ devt2devname (char *devname, int major, int minor)
 	char attr_value[16];
 	char attr_ref_value[16];
 
-	if (sysfs_get_mnt_path (sysfs_path, FILE_NAME_SIZE)) {
-		fprintf (stderr, "-D feature available with sysfs only\n");
-		exit (1);
+	if (sysfs_get_mnt_path(sysfs_path, FILE_NAME_SIZE)) {
+		fprintf(stderr, "-D feature available with sysfs only\n");
+		exit_tool(1);
 	}
 		
-	sprintf (attr_ref_value, "%i:%i\n", major, minor);
-	sprintf (block_path, "%s/block", sysfs_path);
-	sdir = sysfs_open_directory (block_path);
-	sysfs_read_directory (sdir);
+	if(safe_sprintf(attr_ref_value, "%i:%i\n", major, minor)) {
+		fprintf(stderr, "attr_ref_value too small\n");
+		exit_tool(1);
+	}
+	if(safe_sprintf(block_path, "%s/block", sysfs_path)) {
+		fprintf(stderr, "block_path too small\n");
+		exit_tool(1);
+	}
+	sdir = sysfs_open_directory(block_path);
+	sysfs_read_directory(sdir);
 
 	dlist_for_each_data (sdir->subdirs, devp, struct sysfs_directory) {
-		sprintf (attr_path, "%s/%s/dev", block_path, devp->name);
-		sysfs_read_attribute_value (attr_path, attr_value, sizeof(attr_value));
+		if(safe_sprintf(attr_path, "%s/%s/dev",
+				block_path, devp->name)) {
+			fprintf(stderr, "attr_path too small\n");
+			exit_tool(1);
+		}
+		sysfs_read_attribute_value(attr_path, attr_value,
+					   sizeof(attr_value));
 
-		if (!strcmp (attr_value, attr_ref_value)) {
-			sprintf (attr_path, "%s/%s", block_path, devp->name);
-			sysfs_get_name_from_path (attr_path, devname, FILE_NAME_SIZE);
+		if (!strcmp(attr_value, attr_ref_value)) {
+			if(safe_sprintf(attr_path, "%s/%s",
+					block_path, devp->name)) {
+				fprintf(stderr, "attr_path too small\n");
+				exit_tool(1);
+			}
+			sysfs_get_name_from_path(attr_path, devname,
+						 FILE_NAME_SIZE);
 			break;
 		}
 	}
-
-	sysfs_close_directory (sdir);
-	
+	sysfs_close_directory(sdir);
 	return 0;
 }
 
@@ -135,7 +149,11 @@ devinfo (struct path *curpath)
 	/*
 	 * get path prio
 	 */
-	sprintf (buff, "%s /block/%s", conf->default_getprio, curpath->dev);
+	if(safe_sprintf(buff, "%s /block/%s",
+			conf->default_getprio, curpath->dev)) {
+		fprintf(stderr, "buff too small\n");
+		exit_tool(1);
+	}
 
 	dbg("get prio callout :");
 	dbg("==================");
@@ -161,9 +179,11 @@ devinfo (struct path *curpath)
 			 */
 			dbg("get uid callout :");
 			dbg("=================");
-			sprintf (buff, "%s /block/%s",
-				 hwe->getuid, curpath->dev);
-
+			if(safe_sprintf(buff, "%s /block/%s",
+				 hwe->getuid, curpath->dev)) {
+				fprintf(stderr, "buff too small\n");
+				exit_tool(1);
+			}
 			if (execute_program(buff, curpath->wwid,
 			    WWID_SIZE) == 0) {
 				dbg("devinfo found uid : %s", curpath->wwid);
@@ -196,7 +216,11 @@ devinfo (struct path *curpath)
 	 * incidentaly, dealing with this case will make parallel SCSI
 	 * disks treated as 1-path multipaths, which is good : wider audience !
 	 */
-	sprintf (buff, "%s /block/%s", conf->default_getuid, curpath->dev);
+	if(safe_sprintf(buff, "%s /block/%s",
+			conf->default_getuid, curpath->dev)) {
+		fprintf(stderr, "buff too small\n");
+		exit_tool(1);
+	}
 
 	dbg("default get uid callout :");
 	dbg("=========================");
@@ -255,7 +279,10 @@ get_pathvec_sysfs (vector pathvec)
 			return 1;
 		
 		curpath = zalloc(sizeof (struct path));
-		sprintf(curpath->dev, "%s", buff);
+		if(safe_sprintf(curpath->dev, "%s", buff)) {
+			fprintf(stderr, "curpath->dev too small\n");
+			exit_tool(1);
+		}
 
 		if (devinfo(curpath))
 			return 1;
@@ -264,7 +291,10 @@ get_pathvec_sysfs (vector pathvec)
 		free(curpath);
 	}
 		
-	sprintf(path, "%s/block", sysfs_path);
+	if(safe_sprintf(path, "%s/block", sysfs_path)) {
+		fprintf(stderr, "path too small\n");
+		exit_tool(1);
+	}
 	sdir = sysfs_open_directory(path);
 	sysfs_read_directory(sdir);
 
@@ -281,15 +311,16 @@ get_pathvec_sysfs (vector pathvec)
 			if (!strncmp(linkp->name, "device", 6))
 				break;
 		}
-
 		if (linkp == NULL) {
 			continue;
 		}
-
 		basename(devp->path, buff);
 		curpath = zalloc(sizeof(struct path));
-		sprintf(curpath->dev, "%s", buff);
 
+		if(safe_sprintf(curpath->dev, "%s", buff)) {
+			fprintf(stderr, "curpath->dev too small\n");
+			exit_tool(1);
+		}
 		if (devinfo(curpath)) {
 			free (curpath);
 			continue;
@@ -572,6 +603,112 @@ dm_addmap (int task, const char *name, const char *params, unsigned long size) {
 	return 1;
 }
 
+static int
+dm_map_present (char * str)
+{
+        int r = 0;
+	struct dm_task *dmt;
+        struct dm_names *names;
+        unsigned next = 0;
+
+	if (!(dmt = dm_task_create (DM_DEVICE_LIST)))
+		return 0;
+
+	if (!dm_task_run (dmt))
+		goto out;
+
+	if (!(names = dm_task_get_names (dmt)))
+		goto out;
+
+	if (!names->dev) {
+		goto out;
+	}
+
+	do {
+		if (0 == strcmp (names->name, str))
+			r = 1;
+		next = names->next;
+		names = (void *) names + next;
+	} while (next);
+
+	out:
+	dm_task_destroy (dmt);
+	return r;
+}
+
+static int
+dm_get_map(char * name, char * outparams)
+{
+	int r = 0;
+	struct dm_task *dmt;
+	void *next = NULL;
+	uint64_t start, length;
+	char *target_type = NULL;
+	char *params;
+	int cmd;
+
+	cmd = DM_DEVICE_TABLE;
+
+	if (!(dmt = dm_task_create(cmd)))
+		return 0;
+
+	if (!dm_task_set_name(dmt, name))
+		goto out;
+
+	if (!dm_task_run(dmt))
+		goto out;
+
+	/* Fetch 1st target */
+	next = dm_get_next_target(dmt, next, &start, &length,
+				  &target_type, &params);
+
+	if (snprintf(outparams, PARAMS_SIZE, params) >= PARAMS_SIZE)
+		goto out;
+
+	r = 1;
+
+	out:
+	dm_task_destroy(dmt);
+	return r;
+}
+
+static int
+dm_reinstate(char * mapname, char * path)
+{
+        int r = 0, sz = 1, i;
+        struct dm_task *dmt;
+        char *str;
+
+        if (!(dmt = dm_task_create(DM_DEVICE_TARGET_MSG)))
+                return 0;
+
+        if (!dm_task_set_name(dmt, mapname))
+                goto out;
+
+        if (!dm_task_set_sector(dmt, 0))
+                goto out;
+
+	sz = strlen(path) + 10;
+        str = zalloc(sz);
+
+	snprintf(str, sz, "reinstate %s\n", path);
+
+        if (!dm_task_set_message(dmt, str))
+                goto out;
+
+        free(str);
+
+        if (!dm_task_run(dmt))
+                goto out;
+
+        r = 1;
+
+      out:
+        dm_task_destroy(dmt);
+
+        return r;
+}
+
 /*
  * Transforms the path group vector into a proper device map string
  */
@@ -579,6 +716,7 @@ void
 assemble_map (struct multipath * mp)
 {
 	int i, j;
+	int shift, freechar;
 	char * p;
 	vector pgpaths;
 	char * selector;
@@ -626,50 +764,44 @@ assemble_map (struct multipath * mp)
 	}
 
 	p = mp->params;
-	p += sprintf(p, " %i", VECTOR_SIZE(mp->pg));
+	freechar = sizeof(mp->params);
+	
+	shift = snprintf(p, freechar, "%i", VECTOR_SIZE(mp->pg));
 
+	if (shift >= freechar) {
+		fprintf(stderr, "mp->params too small\n");
+		exit_tool(1);
+	}
+	p += shift;
+	freechar -= shift;
+	
 	for (i = 0; i < VECTOR_SIZE(mp->pg); i++) {
 		pgpaths = VECTOR_SLOT(mp->pg, i);
-		p += sprintf(p, " %s %i %i",
-			     selector, VECTOR_SIZE(pgpaths), selector_args);
+		shift = snprintf(p, freechar, " %s %i %i",
+			         selector, VECTOR_SIZE(pgpaths), selector_args);
+		if (shift >= freechar) {
+			fprintf(stderr, "mp->params too small\n");
+			exit_tool(1);
+		}
+		p += shift;
+		freechar -= shift;
 
-		for (j = 0; j < VECTOR_SIZE(pgpaths); j++)
-			p += sprintf(p, " %s",
-				     (char *)VECTOR_SLOT(pgpaths, j));
+		for (j = 0; j < VECTOR_SIZE(pgpaths); j++) {
+			shift = snprintf(p, freechar, " %s",
+					 (char *)VECTOR_SLOT(pgpaths, j));
+			if (shift >= freechar) {
+				fprintf(stderr, "mp->params too small\n");
+				exit_tool(1);
+			}
+			p += shift;
+			freechar -= shift;
+		}
 	}
-}
-
-static int
-map_present (char * str)
-{
-        int r = 0;
-	struct dm_task *dmt;
-        struct dm_names *names;
-        unsigned next = 0;
-
-	if (!(dmt = dm_task_create (DM_DEVICE_LIST)))
-		return 0;
-
-	if (!dm_task_run (dmt))
-		goto out;
-
-	if (!(names = dm_task_get_names (dmt)))
-		goto out;
-
-	if (!names->dev) {
-		goto out;
+	if (freechar < 1) {
+		fprintf(stderr, "mp->params too small\n");
+		exit_tool(1);
 	}
-
-	do {
-		if (0 == strcmp (names->name, str))
-			r = 1;
-		next = names->next;
-		names = (void *) names + next;
-	} while (next);
-
-	out:
-	dm_task_destroy (dmt);
-	return r;
+	snprintf(p, 1, "\n");
 }
 
 static int
@@ -678,11 +810,12 @@ setup_map (vector pathvec, vector mp, int slot)
 	struct path * pp;
 	struct multipath * mpp;
 	int i, iopolicy;
-	char iopolicy_name[32];
+	char iopolicy_name[POLICY_NAME_SIZE];
 	int fd;
 	struct hwentry * hwe = NULL;
 	struct mpentry * mpe;
 	char * mapname;
+	char curparams[PARAMS_SIZE];
 	int op;
 
 	mpp = VECTOR_SLOT(mp, slot);
@@ -780,53 +913,78 @@ setup_map (vector pathvec, vector mp, int slot)
 	if (iopolicy == GROUP_BY_PRIO)
 		group_by_prio (mpp);
 
+	if (mpp->pg == NULL) {
+		dbg("iopolicy failed to produce a ->pg vector");
+		return 1;
+	}
+
+	/*
+	 * transform the mp->pg vector of vectors of paths
+	 * into a mp->params strings to feed the device-mapper
+	 */
+	assemble_map(mpp);
+
+	/*
+	 * select between RELOAD and CREATE
+	 */
+	mapname = mpp->alias ? mpp->alias : mpp->wwid;
+	op = dm_map_present(mapname) ? DM_DEVICE_RELOAD : DM_DEVICE_CREATE;
+	
+	if (conf->verbosity > 1)
+		printf("%s:", (op == DM_DEVICE_RELOAD) ? "reload" : "create");
+	if (conf->verbosity > 0)
+		printf("%s", mapname);
+	if (conf->verbosity > 1)
+		printf(":0 %lu %s %s",
+			mpp->size, DM_TARGET,
+			mpp->params);
+	if (conf->verbosity > 0)
+		printf("\n");
+
+	/*
+	 * last chance to quit before touching the devmaps
+	 */
+	if (conf->dry_run)
+		return 1;
+
 	/*
 	 * device mapper creation or updating
 	 * here we know we'll have garbage on stderr from
 	 * libdevmapper. so shut it down temporarily.
 	 */
-	assemble_map(mpp);
-
-	mapname = mpp->alias ? mpp->alias : mpp->wwid;
-	op = map_present(mpp->wwid) ? DM_DEVICE_RELOAD : DM_DEVICE_CREATE;
-	
 	fd = dup(2);
 	close(2);
 
-	if (op == DM_DEVICE_RELOAD)
-		if (!dm_simplecmd(DM_DEVICE_SUSPEND, mapname)) {
-			dup2(fd, 2);
-			close(fd);
-			return 0;
+	if (op == DM_DEVICE_RELOAD) {
+		if (conf->dev != NULL && filepresent(conf->dev) &&
+		    dm_get_map(mapname, curparams) &&
+		    0 == strncmp(mpp->params, curparams, strlen(mpp->params))) {
+			dm_reinstate(mapname, conf->dev);
+
+			if (conf->verbosity > 1)
+				printf("[reinstate %s in %s]\n",
+					conf->dev, mapname);
+			goto out;
 		}
+		if (!dm_simplecmd(DM_DEVICE_SUSPEND, mapname))
+			goto out;
+	}
 
 	if (dm_addmap(op, mapname, mpp->params, mpp->size)) {
 		dm_simplecmd(DM_DEVICE_REMOVE, mapname);
-		dup2(fd, 2);
-		close(fd);
-		return 0;
+		goto out;
 	}
 
 	if (op == DM_DEVICE_RELOAD)
-		if (!dm_simplecmd(DM_DEVICE_RESUME, mapname)) {
-			dup2(fd, 2);
-			close(fd);
-			return 0;
-		}
-
-	dup2(fd, 2);
-	close(fd);
-
-	if (conf->verbosity > 0)
-		printf ("%s", mapname);
-	if (conf->verbosity > 1)
-		printf (":0 %lu %s%s",
-			mpp->size, DM_TARGET,
-			mpp->params);
-	if (conf->verbosity > 0)
-		printf ("\n");
+		if (!dm_simplecmd(DM_DEVICE_RESUME, mapname))
+			goto out;
 
 	return 1;
+
+	out:
+	dup2(fd, 2);
+	close(fd);
+	return 0;
 }
 
 static void
@@ -856,7 +1014,7 @@ signal_daemon (void)
 
 #define VECTOR_ADDSTR(a, b) \
 	str = zalloc (6 * sizeof(char)); \
-	sprintf (str, b); \
+	snprintf (str, 6, b); \
 	vector_alloc_slot(a); \
 	vector_set_slot(a, str);
 
@@ -880,10 +1038,10 @@ setup_default_blist (vector blist)
 
 #define VECTOR_ADDHWE(a, b, c, d, e) \
 	hwe = zalloc (sizeof(struct hwentry)); \
-	hwe->vendor = zalloc (9 * sizeof(char)); \
-	sprintf (hwe->vendor, b); \
-	hwe->product = zalloc (17 * sizeof(char)); \
-	sprintf (hwe->product, c); \
+	hwe->vendor = zalloc (SCSI_VENDOR_SIZE * sizeof(char)); \
+	snprintf (hwe->vendor, SCSI_VENDOR_SIZE, b); \
+	hwe->product = zalloc (SCSI_PRODUCT_SIZE * sizeof(char)); \
+	snprintf (hwe->product, SCSI_PRODUCT_SIZE, c); \
 	hwe->iopolicy = d; \
 	hwe->getuid = e; \
 	vector_alloc_slot(a); \
@@ -1108,12 +1266,6 @@ main (int argc, char *argv[])
 		print_all_mp(mp);
 	}
 
-	/*
-	 * last chance to quit before messing with devmaps
-	 */
-	if (conf->dry_run)
-		exit_tool(0);
-
 	if (conf->verbosity > 1)
 		fprintf (stdout, "#\n# device maps :\n#\n");
 
@@ -1124,7 +1276,7 @@ main (int argc, char *argv[])
 	 * signal multipathd that new devmaps may have come up
 	 */
 	if (conf->signal)
-		signal_daemon ();
+		signal_daemon();
 	
 	/*
 	 * free allocs
