@@ -5,6 +5,7 @@
 #include <fcntl.h>
 #include <sys/ioctl.h>
 #include "libsysfs/sysfs/libsysfs.h"
+#include "log.h"
 
 #include "devinfo.h"
 
@@ -30,20 +31,29 @@ get_lun_strings(char * vendor_id, char * product_id, char * rev, char * devname)
 	if (sysfs_get_mnt_path (sysfs_path, FILE_NAME_SIZE))
 		return 1;
 		
-	sprintf(attr_path, "%s/block/%s/device/vendor",
-		sysfs_path, devname);
+	if(safe_sprintf(attr_path, "%s/block/%s/device/vendor",
+		sysfs_path, devname)) {
+		fprintf(stderr, "get_lun_strings: attr_path too small\n");
+		return 1;
+	}
 	if (0 > sysfs_read_attribute_value(attr_path,
 		attr_buff, 17)) return 0;
 	memcpy (vendor_id, attr_buff, 8);
 
-	sprintf(attr_path, "%s/block/%s/device/model",
-		sysfs_path, devname);
+	if(safe_sprintf(attr_path, "%s/block/%s/device/model",
+		sysfs_path, devname)) {
+		fprintf(stderr, "get_lun_strings: attr_path too small\n");
+		return 1;
+	}
 	if (0 > sysfs_read_attribute_value(attr_path,
 		attr_buff, 17)) return 0;
 	memcpy (product_id, attr_buff, 16);
  
-	sprintf(attr_path, "%s/block/%s/device/rev",
-		sysfs_path, devname);
+	if(safe_sprintf(attr_path, "%s/block/%s/device/rev",
+		sysfs_path, devname)) {
+		fprintf(stderr, "get_lun_strings: attr_path too small\n");
+		return 1;
+	}
 	if (0 > sysfs_read_attribute_value(attr_path,
 		attr_buff, 17)) return 0;
 	memcpy (rev, attr_buff, 4);
